package java.text;

import java.util.HashMap;
import java.io.InvalidObjectException;
import java.util.Map;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.util.TimeZone;
import sun.util.locale.provider.LocaleServiceProviderPool;
import java.text.spi.DateFormatProvider;
import java.util.Locale;
import java.util.Date;
import java.util.Calendar;

public abstract class DateFormat extends Format
{
    protected Calendar calendar;
    protected NumberFormat numberFormat;
    public static final int ERA_FIELD = 0;
    public static final int YEAR_FIELD = 1;
    public static final int MONTH_FIELD = 2;
    public static final int DATE_FIELD = 3;
    public static final int HOUR_OF_DAY1_FIELD = 4;
    public static final int HOUR_OF_DAY0_FIELD = 5;
    public static final int MINUTE_FIELD = 6;
    public static final int SECOND_FIELD = 7;
    public static final int MILLISECOND_FIELD = 8;
    public static final int DAY_OF_WEEK_FIELD = 9;
    public static final int DAY_OF_YEAR_FIELD = 10;
    public static final int DAY_OF_WEEK_IN_MONTH_FIELD = 11;
    public static final int WEEK_OF_YEAR_FIELD = 12;
    public static final int WEEK_OF_MONTH_FIELD = 13;
    public static final int AM_PM_FIELD = 14;
    public static final int HOUR1_FIELD = 15;
    public static final int HOUR0_FIELD = 16;
    public static final int TIMEZONE_FIELD = 17;
    private static final long serialVersionUID = 7218322306649953788L;
    public static final int FULL = 0;
    public static final int LONG = 1;
    public static final int MEDIUM = 2;
    public static final int SHORT = 3;
    public static final int DEFAULT = 2;
    
    @Override
    public final StringBuffer format(Object o, StringBuffer sb, FieldPosition fieldPosition) {
        if ((o instanceof Date)) {
            return this.format((Date)o, sb, fieldPosition);
        }
        if ((o instanceof Number)) {
            return this.format(new Date(((Number)o).longValue()), sb, fieldPosition);
        }
        throw new IllegalArgumentException("Cannot format given Object as a Date");
    }
    
    public abstract StringBuffer format(Date p0, StringBuffer p1, FieldPosition p2);
    
    public final String format(Date date) {
        return this.format(date, new StringBuffer(), DontCareFieldPosition.INSTANCE).toString();
    }
    
    public Date parse(String str) throws ParseException {
        ParsePosition parsePosition = new ParsePosition(0);
        Date parse = this.parse(str, parsePosition);
        if (parsePosition.index == 0) {
            throw new ParseException("Unparseable date: \"" + str + "\"", parsePosition.errorIndex);
        }
        return parse;
    }
    
    public abstract Date parse(String p0, ParsePosition p1);
    
    @Override
    public Object parseObject(String s, ParsePosition parsePosition) {
        return this.parse(s, parsePosition);
    }
    
    public static final DateFormat getTimeInstance() {
        return get(2, 0, 1, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getTimeInstance(int n) {
        return get(n, 0, 1, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getTimeInstance(int n, Locale locale) {
        return get(n, 0, 1, locale);
    }
    
    public static final DateFormat getDateInstance() {
        return get(0, 2, 2, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getDateInstance(int n) {
        return get(0, n, 2, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getDateInstance(int n, Locale locale) {
        return get(0, n, 2, locale);
    }
    
    public static final DateFormat getDateTimeInstance() {
        return get(2, 2, 3, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getDateTimeInstance(int n, int n2) {
        return get(n2, n, 3, Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static final DateFormat getDateTimeInstance(int n, int n2, Locale locale) {
        return get(n2, n, 3, locale);
    }
    
    public static final DateFormat getInstance() {
        return getDateTimeInstance(3, 3);
    }
    
    public static Locale[] getAvailableLocales() {
        return LocaleServiceProviderPool.getPool(DateFormatProvider.class).getAvailableLocales();
    }
    
    public void setCalendar(Calendar calendar) {
        this.calendar = calendar;
    }
    
    public Calendar getCalendar() {
        return this.calendar;
    }
    
    public void setNumberFormat(NumberFormat numberFormat) {
        this.numberFormat = numberFormat;
    }
    
    public NumberFormat getNumberFormat() {
        return this.numberFormat;
    }
    
    public void setTimeZone(TimeZone timeZone) {
        this.calendar.setTimeZone(timeZone);
    }
    
    public TimeZone getTimeZone() {
        return this.calendar.getTimeZone();
    }
    
    public void setLenient(boolean lenient) {
        this.calendar.setLenient(lenient);
    }
    
    public boolean isLenient() {
        return this.calendar.isLenient();
    }
    
    @Override
    public int hashCode() {
        return this.numberFormat.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || this.getClass() != o.getClass()) {
            return false;
        }
        DateFormat dateFormat = (DateFormat)o;
        return this.calendar.getFirstDayOfWeek() == dateFormat.calendar.getFirstDayOfWeek() && this.calendar.getMinimalDaysInFirstWeek() == dateFormat.calendar.getMinimalDaysInFirstWeek() && this.calendar.isLenient() == dateFormat.calendar.isLenient() && this.calendar.getTimeZone().equals(dateFormat.calendar.getTimeZone()) && this.numberFormat.equals(dateFormat.numberFormat);
    }
    
    @Override
    public Object clone() {
        DateFormat dateFormat = (DateFormat)super.clone();
        dateFormat.calendar = (Calendar)this.calendar.clone();
        dateFormat.numberFormat = (NumberFormat)this.numberFormat.clone();
        return dateFormat;
    }
    
    private static DateFormat get(int i, int j, int n, Locale locale) {
        if ((n & 0x1) != 0x0) {
            if (i < 0 || i > 3) {
                throw new IllegalArgumentException("Illegal time style " + i);
            }
        }
        else {
            i = -1;
        }
        if ((n & 0x2) != 0x0) {
            if (j < 0 || j > 3) {
                throw new IllegalArgumentException("Illegal date style " + j);
            }
        }
        else {
            j = -1;
        }
        DateFormat dateFormat = get(LocaleProviderAdapter.getAdapter(DateFormatProvider.class, locale), i, j, locale);
        if (dateFormat == null) {
            dateFormat = get(LocaleProviderAdapter.forJRE(), i, j, locale);
            return dateFormat;
        }
        return dateFormat;
    }
    
    private static DateFormat get(LocaleProviderAdapter localeProviderAdapter, int n, int n2, Locale locale) {
        DateFormatProvider dateFormatProvider = localeProviderAdapter.getDateFormatProvider();
        DateFormat dateFormat;
        if (n == -1) {
            dateFormat = dateFormatProvider.getDateInstance(n2, locale);
        }
        else if (n2 == -1) {
            dateFormat = dateFormatProvider.getTimeInstance(n, locale);
        }
        else {
            dateFormat = dateFormatProvider.getDateTimeInstance(n2, n, locale);
        }
        return dateFormat;
    }
    
    protected DateFormat() {
    }
    
    public static class Field extends Format.Field
    {
        private static final long serialVersionUID = 7441350119349544720L;
        private static final Map<String, DateFormat.Field> instanceMap;
        private static final DateFormat.Field[] calendarToFieldMapping;
        private int calendarField;
        public static final DateFormat.Field ERA;
        public static final DateFormat.Field YEAR;
        public static final DateFormat.Field MONTH;
        public static final DateFormat.Field DAY_OF_MONTH;
        public static final DateFormat.Field HOUR_OF_DAY1;
        public static final DateFormat.Field HOUR_OF_DAY0;
        public static final DateFormat.Field MINUTE;
        public static final DateFormat.Field SECOND;
        public static final DateFormat.Field MILLISECOND;
        public static final DateFormat.Field DAY_OF_WEEK;
        public static final DateFormat.Field DAY_OF_YEAR;
        public static final DateFormat.Field DAY_OF_WEEK_IN_MONTH;
        public static final DateFormat.Field WEEK_OF_YEAR;
        public static final DateFormat.Field WEEK_OF_MONTH;
        public static final DateFormat.Field AM_PM;
        public static final DateFormat.Field HOUR1;
        public static final DateFormat.Field HOUR0;
        public static final DateFormat.Field TIME_ZONE;
        
        public static DateFormat.Field ofCalendarField(int i) {
            if (i < 0 || i >= DateFormat.Field.calendarToFieldMapping.length) {
                throw new IllegalArgumentException("Unknown Calendar constant " + i);
            }
            return DateFormat.Field.calendarToFieldMapping[i];
        }
        
        protected Field(String s, int calendarField) {
            super(s);
            this.calendarField = calendarField;
            if (this.getClass() == DateFormat.Field.class) {
                DateFormat.Field.instanceMap.put(s, this);
                if (calendarField >= 0) {
                    DateFormat.Field.calendarToFieldMapping[calendarField] = this;
                }
            }
        }
        
        public int getCalendarField() {
            return this.calendarField;
        }
        
        @Override
        protected Object readResolve() throws InvalidObjectException {
            if (this.getClass() != DateFormat.Field.class) {
                throw new InvalidObjectException("subclass didn't correctly implement readResolve");
            }
            Object value = DateFormat.Field.instanceMap.get(this.getName());
            if (value != null) {
                return value;
            }
            throw new InvalidObjectException("unknown attribute name");
        }
        
        static {
            instanceMap = new HashMap(18);
            calendarToFieldMapping = new DateFormat.Field[17];
            ERA = new DateFormat.Field("era", 0);
            YEAR = new DateFormat.Field("year", 1);
            MONTH = new DateFormat.Field("month", 2);
            DAY_OF_MONTH = new DateFormat.Field("day of month", 5);
            HOUR_OF_DAY1 = new DateFormat.Field("hour of day 1", -1);
            HOUR_OF_DAY0 = new DateFormat.Field("hour of day", 11);
            MINUTE = new DateFormat.Field("minute", 12);
            SECOND = new DateFormat.Field("second", 13);
            MILLISECOND = new DateFormat.Field("millisecond", 14);
            DAY_OF_WEEK = new DateFormat.Field("day of week", 7);
            DAY_OF_YEAR = new DateFormat.Field("day of year", 6);
            DAY_OF_WEEK_IN_MONTH = new DateFormat.Field("day of week in month", 8);
            WEEK_OF_YEAR = new DateFormat.Field("week of year", 3);
            WEEK_OF_MONTH = new DateFormat.Field("week of month", 4);
            AM_PM = new DateFormat.Field("am pm", 9);
            HOUR1 = new DateFormat.Field("hour 1", -1);
            HOUR0 = new DateFormat.Field("hour", 10);
            TIME_ZONE = new DateFormat.Field("time zone", -1);
        }
    }
}
